SystemOrganization addCategory: #ConstraintsDebugger!

!ContextPart methodsFor: '*ConstraintsDebugger' stamp: 'tfel 10/4/2014 11:24'!
send: selector super: superFlag numArgs: numArgs
	"redirect sends to alwaysSolveWith: to alwaysSolveWithDebug: so we can step into constraint construction"
	<layer: #ccmDebug>
	^ selector = #alwaysSolveWith:
		ifTrue: [self proceed: ConstraintDebugger DebugSelector with: superFlag with: numArgs]
		ifFalse: [self proceed: selector with: superFlag with: numArgs]! !

Debugger subclass: #ConstraintDebugger
	instanceVariableNames: 'constraintIndex constraintLayer'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ConstraintsDebugger'!

!ConstraintDebugger class methodsFor: 'constants' stamp: 'tfel 10/4/2014 11:23'!
DebugSelector

	^ #alwaysSolveWithDebug:! !

!ConstraintDebugger class methodsFor: 'examples' stamp: 'tfel 7/25/2014 11:22'!
example1
	"Select the below code and then use the yellow-button menu to 'debug it as constraint'. When asked which solver to use, select 'newCassowarySolver'"
	| ex |
	ex := ExampleClass new.
	ex arrayCollection: #(1 2 0).
	ex arrayCollection allSatisfy: [:each | each >= 3].! !

!ConstraintDebugger methodsFor: 'toolbuilder' stamp: 'tfel 10/4/2014 11:59'!
buildFullWith: builder
	| windowSpec listSpec textSpec extent |
	windowSpec := builder pluggableWindowSpec new.
	windowSpec
		model: self;
		label: #labelString.
	Display height < 800 "a small screen" 
		ifTrue:[extent := RealEstateAgent standardWindowExtent]
		ifFalse:[extent := 600@700].
	windowSpec extent: extent.
	windowSpec children: OrderedCollection new.

	listSpec := builder pluggableListSpec new.
	listSpec 
		model: self;
		list: #contextStackList; 
		getIndex: #contextStackIndex; 
		setIndex: #toggleContextStackIndex:; 
		menu: #contextStackMenu:shifted:; 
		keyPress: #contextStackKey:from:;
		frame: (0@0 corner: 0.6@0.22).
	windowSpec children add: listSpec.

	listSpec := builder pluggableListSpec new.
	listSpec 
		model: self;
		list: #constraintList; 
		getIndex: #constraintIndex; 
		setIndex: #setConstraintIndex:; 
		menu: #constraintMenu:shifted:; 
		frame: (0.6@0 corner: 1@0.22).
	windowSpec children add: listSpec.

	textSpec := self buildCodePaneWith: builder.
	textSpec frame: (0@0.22corner: 1@0.8).
	windowSpec children add: textSpec.

	listSpec := builder pluggableListSpec new.
	listSpec 
		model: self receiverInspector;
		list: #fieldList; 
		getIndex: #selectionIndex; 
		setIndex: #toggleIndex:; 
		menu: #fieldListMenu:; 
		keyPress: #inspectorKey:from:;
		frame: (0@0.8 corner: 0.2@1).
	windowSpec children add: listSpec.

	textSpec := builder pluggableTextSpec new.
	textSpec 
		model: self receiverInspector;
		getText: #contents; 
		setText: #accept:; 
		selection: #contentsSelection; 
		menu: #codePaneMenu:shifted:;
		frame: (0.2@0.8 corner: 0.5@1).
	windowSpec children add: textSpec.

	listSpec := builder pluggableListSpec new.
	listSpec 
		model: self contextVariablesInspector;
		list: #fieldList; 
		getIndex: #selectionIndex; 
		setIndex: #toggleIndex:; 
		menu: #fieldListMenu:; 
		keyPress: #inspectorKey:from:;
		frame: (0.5@0.8 corner: 0.7@1).
	windowSpec children add: listSpec.

	textSpec := builder pluggableTextSpec new.
	textSpec 
		model: self contextVariablesInspector;
		getText: #contents; 
		setText: #accept:; 
		selection: #contentsSelection; 
		menu: #codePaneMenu:shifted:;
		frame: (0.7@0.8 corner: 1@1).
	windowSpec children add: textSpec.

	^ [builder build: windowSpec] ensure: [self labelString: 'Constraint Debugger'.]! !

!ConstraintDebugger methodsFor: 'constraints-access' stamp: 'tfel 10/4/2014 12:00'!
constraintChanged

	self constraintLayer
		ifEmpty: [
			(contextStack size > 1 and: [
				contextStack second selector = self class DebugSelector and: [
					contextStack first closure = contextStack second receiver]]) ifTrue: [
						"stepping into a constraint"
						self constraintLayer: #ccm.
						self labelString: self labelString, ' (in ccm mode)']]
		ifNotEmpty: [
	.		(contextStack allButFirst "first might be the alwaysSolveWithDebug: method"
					noneSatisfy: [:c | c selector = self class DebugSelector]) ifTrue: [
				"we left the constraint, deactivate ccm"
				self constraintLayer: #''.
				self labelString: (self labelString copyReplaceAll: ' (in ccm mode)' with: '')]].
	self changed: #constraintList.! !

!ConstraintDebugger methodsFor: 'constraints-access' stamp: 'tfel 7/25/2014 10:58'!
constraintIndex

	^ constraintIndex ifNil: [constraintIndex := 0]! !

!ConstraintDebugger methodsFor: 'accessing' stamp: 'tfel 10/4/2014 10:54'!
constraintIndex: anObject

	constraintIndex := anObject! !

!ConstraintDebugger methodsFor: 'accessing' stamp: 'tfel 10/4/2014 11:27'!
constraintLayer

	^ constraintLayer ifNil: [constraintLayer := #'']! !

!ConstraintDebugger methodsFor: 'accessing' stamp: 'tfel 10/4/2014 10:54'!
constraintLayer: anObject

	constraintLayer := anObject! !

!ConstraintDebugger methodsFor: 'constraints-access' stamp: 'tfel 10/4/2014 11:25'!
constraintList
	| co |
	co := (Processor activeProcess environmentAt: #constraintObject ifAbsent: [^ #()]).
	self flag: #XXX.
	(co respondsTo: #externalConstraints)
		ifTrue: [^ co externalConstraints]
		ifFalse: [^ co expressions]! !

!ConstraintDebugger methodsFor: 'constraints-access' stamp: 'tfel 7/25/2014 11:12'!
constraintMenu: aMenu shifted: shifted

	^ aMenu addList: {
			{'remove'. #removeSelectedConstraint}.
			#-.
			{'explore'. #exploreSelectedConstraint}.
		}! !

!ConstraintDebugger methodsFor: 'accessing' stamp: 'tfel 10/4/2014 11:57'!
contents: aText notifying: aController

	^ #ccmDebug withLayerDo: [
		[self constraintLayer withLayerDo: [
			super contents: aText notifying: aController]] ensure: [self constraintChanged]]! !

!ConstraintDebugger methodsFor: 'context stack menu' stamp: 'tfel 10/4/2014 11:16'!
doStep

	^ #ccmDebug withLayerDo: [[
			self constraintLayer withLayerDo: [super doStep]] ensure: [self constraintChanged]]! !

!ConstraintDebugger methodsFor: 'constraints-access' stamp: 'tfel 7/25/2014 11:12'!
exploreSelectedConstraint
	| co |
	co := (Processor activeProcess environmentAt: #constraintObject).
	self flag: #XXX.
	((co respondsTo: #externalConstraints)
		ifTrue: [co externalConstraints at: self constraintIndex]
		ifFalse: [co expressions at: self constraintIndex]) explore.! !

!ConstraintDebugger methodsFor: 'private' stamp: 'tfel 10/4/2014 11:56'!
process: aProcess controller: aController context: aContext
	"| aConstraintSolver constraint solvers |
	solvers := ConstraintSolver class organization listAtCategoryNamed: #'instance creation'.
	aConstraintSolver := ConstraintSolver perform: (UIManager default chooseFrom: solvers values: solvers title: 'Which solver?').
	constraint := aConstraintSolver class constraintClass new
					constraintBlock: self;
					priority: ConstraintPriority required;
					solver: aConstraintSolver;
					yourself.
	Processor activeProcess environmentAt: #constraintObject put: constraint."
	^ super process: aProcess controller: aController context: aContext! !

!ConstraintDebugger methodsFor: 'constraints-access' stamp: 'tfel 7/25/2014 11:09'!
removeSelectedConstraint
	| co |
	co := (Processor activeProcess environmentAt: #constraintObject).
	self flag: #XXX.
	(co respondsTo: #externalConstraints)
		ifTrue: [co externalConstraints removeAt: self constraintIndex]
		ifFalse: [co expressions removeAt: self constraintIndex].
	self changed: #constraintList.
	self changed: #constraintIndex.! !

!ConstraintDebugger methodsFor: 'context stack menu' stamp: 'tfel 10/4/2014 11:16'!
restart

	^ #ccmDebug withLayerDo: [
		[self constraintLayer withLayerDo: [super restart]] ensure: [self constraintChanged]]! !

!ConstraintDebugger methodsFor: 'context stack menu' stamp: 'tfel 10/4/2014 11:16'!
returnValue

	^ #ccmDebug withLayerDo: [[
		self constraintLayer withLayerDo: [super returnValue]] ensure: [self constraintChanged]]! !

!ConstraintDebugger methodsFor: 'code pane menu' stamp: 'tfel 10/4/2014 11:16'!
runUntil

	^ #ccmDebug withLayerDo: [
		[self constraintLayer withLayerDo: [super runUntil]] ensure: [self constraintChanged]]! !

!ConstraintDebugger methodsFor: 'context stack menu' stamp: 'tfel 10/4/2014 11:47'!
send
	
	^ #ccmDebug withLayerDo: [
		[self constraintLayer withLayerDo: [super send]] ensure: [self constraintChanged]]! !

!ConstraintDebugger methodsFor: 'constraints-access' stamp: 'tfel 7/25/2014 11:06'!
setConstraintIndex: aNumber

	constraintIndex := aNumber.
	self changed: #constraintIndex.! !

!ConstraintDebugger methodsFor: 'context stack menu' stamp: 'tfel 10/4/2014 11:47'!
stepIntoBlock
	
	^ #ccmDebug withLayerDo: [
		[self constraintLayer withLayerDo: [super stepIntoBlock]] ensure: [self constraintChanged]]! !

!ConstraintDebugger methodsFor: 'initialize' stamp: 'tfel 7/25/2014 10:41'!
windowIsClosing

	Processor activeProcess environmentRemoveKey: #constraintObject ifAbsent: [].
	super windowIsClosing.! !
